#!/usr/bin/env node
'use strict';

const path = require('path');

// Create MIO object
global.mio = {};

// Detect root path
global.mio.appPath = path.dirname(require.main.filename);

// Init config
const config = require('./services/config')().get();
if(!global.mio.config){
  global.mio.config = config;
}

// Include utils
var mio = require('./utils/utils');

// Init Services Runner
const service = require('./services/moduleComposer');

// Start Server
mio.server = function(options){
  // Init MongoDB
  if(options && options.mongoose){
    require('./services/mongoose').init(options.mongoose);
  }

  // Run cluster
  require('./services/cluster').init();
};

// Start Module as a Service
mio.service = function(serviceName, options){
  return service.start(serviceName, options);
};

// List available modules
mio.listServices = function(){
  return service.listExternalModules();
};

// Init Mongoose only
mio.initMongo = function(mongoose){
  require('./services/mongoose').init(mongoose);
};

module.exports = mio;
