// Cluster service
'use strict';

const cluster = require('cluster');
const chalk = require('chalk');
const winston = require('winston');

// Worker process
function createWorker(){

  // Init modules
  require('./moduleComposer').loadModules();

  // Init HTTP server
  require('./express');

  // Project Ready Message
  printOnce(()=>{
    winston.info('-'.repeat(70));
    winston.info(chalk.blue('Server Ready'));
    winston.info('='.repeat(70));
  });
}

// Init cluster
function init(){

  // Code to run if we're in the master process
  if (cluster.isMaster) {

    // Intro message
    winston.info('='.repeat(70));
    winston.info('Just', chalk.green('Make It Once'),'|',chalk.blue('MakeItOnce.net'));
    winston.info('-'.repeat(70));
    winston.info('Starting',global.name,'v'+global.version,'app server...');

    // Count the machine's CPUs
    var cpuCount = require('os').cpus().length;
    winston.info('Detected '+chalk.green(cpuCount)+' cores');

    if(config.useCluster){
      // Create a worker for each CPU
      for (var i = 0; i < cpuCount; i += 1) {
        cluster.fork();
      }

      // Listen for dying workers
      cluster.on('exit', function (worker) {
        // Replace the dead worker, we're not sentimental
        winston.error('Worker ' + worker.id + ' died :( Restarting...');
        cluster.fork();

      });
    }else{
      // Run in single core mode
      createWorker();
    }

  // Code to run if we're in a worker process
  }else{
    // Start worker for each core
    createWorker();
  }
}

// Guard for printing message only once
function printOnce(callback){
  if(workerId()==1 && callback){
    callback();
  }
}

// Return Worker Id
function workerId(){
  if(cluster.isMaster){
    return 1;
  }else if(cluster.worker && cluster.worker.id){
    return cluster.worker.id;
  }
  return -1;
}

module.exports = {
  init: init,
  printOnce: printOnce,
  workerId: workerId,
  createWorker: createWorker
};
